function [pstat,params,gsize,vmu]=fanova1w(data,gparams)
%% [pstat,params,gsize, vmu]=fanova1w(data,gparams)
%% Homogeneous One-way ANOVA for functional data
%%  
%%
%%  data=[A,y1,y2,...,yp]: Nx(p+1)  data matrix
%%                           A: indicator for Factor A
%%   data matrx [y1,y2,...,yp]  each row = a discretized version of a function                        
%%
%%   gparams=[method,apprflag,selflag];
%%%% 
%%   method = 1    L2-norm based test
%%          = 2    F-type test
%%          = 3    Globalizing the pointwise F-test
%%          = 4    Pointwise F-test
%%          = 5    using sup Fn(t) as test statistic, bootstrapping
%%   if method<=4,
%%   apprflag = 0  naive method
%%            = 1  bias-reduced method
%%   if selflag==0, using all points
%%             ==1, using selected points
%%   if method=5,
%%   Nboot=apprflag;
%%       if not specified, Nboot=10000 by default
%% pstat=[stat,pvalue]  if method<=3,
%%      =[stat(:), pvalue(:)]  if method==4
%%      =[stat, pvalue, crit]  if mehod==5
%% params=  [beta, (k-1)*kappa,kappa];  when method =1 or 3
%%          [(k-1)*kappa,(N-k)*kappa,kappa];  when method =2
%%          [k-1,N-k] when method =4
%%          bstat  when method==5
%% gsize---sample sizes of each cell
%% vmu=[mu1;mu2;...mua];
%% Jin-Ting Zhang
%% April 27, 2010
%% Revised April 18, 2011
%% Revised Oct 04, 2011
%% NUS, Singapore

[N,p]=size(data);
aflag=data(:,1);
yy=data(:,2:p);
p=p-1; %% data dimension
aflag0=unique(aflag); %% Levels of Factor A
k=length(aflag0); %% Number of Factor A's levels
if nargin<2|length(gparams)==0,
    method =2; %% F-type   test
    apprflag=0; %% naive method
    selflag=0;
elseif length(gparams)==1,
    method=gparams(1);
    if method<=4,
        apprflag=0;
    else
        Nboot=10000;
    end
    selflag=0;
elseif length(gparams)==2,
   method=gparams(1);
   if method<=4,
       apprflag=gparams(2); 
   else
       Nboot=gparams(2);
   end
   selflag=0;
   
   elseif length(gparams)==3,
   method=gparams(1);
   if method<=4,
       apprflag=gparams(2); 
   else
       Nboot=gparams(2);
   end
   selflag=gparams(3);
end


%% computing pointwise SSR, SSE 
mu0=mean(yy); %% sample grand mean function
gsize=[];SSR=0;SSE=0;vmu=[]; V=[];
for i=1:k,
    iflag=(aflag==aflag0(i));yyi=yy(iflag,:);
    ni=sum(iflag);gsize=[gsize;ni];
    mui=mean(yyi);
    SSR=SSR+ni*(mui-mu0).^2;  %% pointwise SSR
    Vi=yyi-ones(ni,1)*mui;
    SSE=SSE+sum(Vi.^2);  %% pointwise SSE
    vmu=[vmu;mui];  %%kxm  group mean vectors
    V=[V;Vi];       %%Nxm  subject-effect vectors N=n1+n2+...+nk
end

if method<=2,
    %% selecting the points
    if selflag==1,
       Fv=(SSR*(N-k))/(SSE*(k-1));
       selpts=(Fv>finv(.95,k-1,N-k));
       
       if sum(selpts)>1,
           
          SSR=SSR(:,selpts);
          SSE=SSE(:,selpts);
          V=V(:,selpts);
          p=size(V,2);
       end
       
    end
    
    
if N>p,
    V=V'*V/(N-k);  %% pxp pooled covariance matrix
else
    V=V*V'/(N-k);  %% Nx N matrix whch has the same non-zero eigen values with Sigma
end

A=trace(V);B=trace(V^2);
if apprflag==0, %% the naive method
    A2=A^2;B2=B;
elseif apprflag==1, %% the bias-reduced method
    A2=(N-k)*(N-k+1)/(N-k-1)/(N-k+2)*(A^2-2*B/(N-k+1));
    B2=(N-k)^2/(N-k-1)/(N-k+2)*(B-A^2/(N-k));
end

if method==1,%% L2-norm based test
    stat=sum(SSR);
    beta=B2/A;kappa=A2/B2;
    pvalue=1-chi2cdf(stat/beta,(k-1)*kappa);
    pstat=[stat,pvalue]; params=[beta,(k-1)*kappa,kappa];
elseif method==2, %% F-type test
    stat=sum(SSR)/sum(SSE)*(N-k)/(k-1);
    kappa=A2/B2;
    pvalue=1-fcdf(stat,(k-1)*kappa, (N-k)*kappa);
    pstat=[stat,pvalue]; params=[(k-1)*kappa,(N-k)*kappa,kappa];
end

end %% for if method<=2

if method==3, %% %% Globalizing the pointwise F-test
    %V=V'*V/(N-k);  %% covariance matrix
    %dd=diag(V).^(-1);
    dd=sqrt(sum(V.^2));
    V=V./(ones(N,1)*dd);
    if N>=p,
        V=V'*V;
    else
        V=V*V';
    end
    
    A=(N-k)/(N-k-2);
    %B=dd'*V.^2*dd/p^2/(k-1);
    B=trace(V^2)/p^2/(k-1);
    
    stat=mean(SSR./SSE*(N-k)/(k-1));
    beta=B/A;d=A^2/B;
    pvalue=1-chi2cdf(stat/beta,d);
    pstat=[stat,pvalue]; params=[beta,d,d/(k-1)];
end

if method==4,%% Pointwise F-test
   stat=SSR./SSE*(N-k)/(k-1);
   pvalue=1-fcdf(stat,k-1,N-k);
   pstat=[stat(:),pvalue(:)];
   params=[k-1,N-k];
end

if method==5,%% using sup Fn(t), by bootstrapping
   stat=max(SSR./SSE*(N-k)/(k-1));  %%extreme F-test statistic
   %% Bootstrapping the P-value and critical value
   for ii=1:Nboot,
        %%Bootstrapping
        bflag=fix(rand(N,1)*N)+1;
        bV=V(bflag,:);
       bSSR=0;bSSE=0;bmu0=mean(bV);
       cn=cumsum(gsize);cn=[0;cn];
       for i=1:k,
         iflag=[(cn(i)+1):cn(i+1)];bvi=bV(iflag,:);
          ni=gsize(i);bmui=mean(bvi);
          bSSR=bSSR+ni*(bmui-bmu0).^2;  %% pointwise SSR
          bVi=bvi-ones(ni,1)*bmui;
          bSSE=bSSE+sum(bVi.^2);  %% pointwise SSE
       end
       bstat(ii)=max(bSSR./bSSE*(N-k)/(k-1));
   end  %% end of bootstrapping
   
   bstat0=sort(bstat); crit=bstat0(fix(.95*Nboot));
   pvalue=mean(bstat>stat);
   pstat=[stat,pvalue,crit];
   params=bstat(:);
end




    
